//#define BLYNK_PRINT Serial
//#define BLYNK_DEBUG

#include <ESP8266WiFi.h>
#include <BlynkSimpleEsp8266.h>
#include <TimeLib.h>
#include <WidgetRTC.h>
#include <SPI.h>
#include <MFRC522.h>

char auth[] = "YourAuthToken";
char ssid[] = "YourNetworkName";
char pass[] = "YourPassword";

byte RFID_KEY[]    = {
  0x61, 0x70, 0x6F, 0x73,
  0x74, 0x65, 0x72, 0x69
};

#define RFID_BLOCK_ADDR 7
#define INT_LED D4
#define LOCK_PIN D0
#define RED_PIN D1
#define RST_PIN D3
#define SS_PIN D8

MFRC522 mfrc522(SS_PIN, RST_PIN);
BlynkTimer timer;
WidgetRTC rtc;

int rowIndex = 0;
boolean unlock = false;

void dump_byte_array(byte *buffer, byte bufferSize) {
  for (byte i = 0; i < bufferSize; i++) {
    Serial.print(buffer[i] < 0x10 ? " 0" : " ");
    Serial.print(buffer[i], HEX);
  }
  Serial.println("");
}

void recordUnlock(String unlockType)
{
  Serial.println("recordUnlock");
  String currentDateTime = String(hour()) + ":" + minute() + ":" + second() + " " + day() + "/" + month() + "/" + year();
  Blynk.virtualWrite(V1, "add", rowIndex, currentDateTime, unlockType);
  Blynk.virtualWrite(V1, "pick", rowIndex);
  rowIndex++;
}

void lockDoor()
{
  Serial.println("lock");
  unlock = false;
  digitalWrite(LOCK_PIN, LOW);
  digitalWrite(RED_PIN, HIGH);
}

void unlockDoor(String unlockType)
{
  Serial.println("unlock");
  unlock = true;
  recordUnlock(unlockType);
  digitalWrite(LOCK_PIN, HIGH);
  digitalWrite(RED_PIN, LOW);
  timer.setTimeout(5*1000, lockDoor);
}

BLYNK_WRITE(V0)
{
  if (param.asInt() == 1 && unlock == false){
    unlockDoor("Phone");
  }
}

BLYNK_CONNECTED() {
  rtc.begin();
}

void setup()
{
  Serial.begin(1000000);
//  Serial.begin(115200);
  
  pinMode(INT_LED, OUTPUT);
  digitalWrite(INT_LED, HIGH);

  pinMode(LOCK_PIN, OUTPUT);
  digitalWrite(LOCK_PIN, LOW);

  pinMode(RED_PIN, OUTPUT);
  digitalWrite(RED_PIN, HIGH);

  SPI.begin();
  mfrc522.PCD_Init();
  
  Blynk.begin(auth, ssid, pass);
  setSyncInterval(10 * 60);

  Blynk.virtualWrite(V1, "clr");
}

void loop()
{ 
  if (mfrc522.PICC_IsNewCardPresent() && unlock == false) {
    if (mfrc522.PICC_ReadCardSerial()) {
      Serial.println("UID and Type : ");
      dump_byte_array(mfrc522.uid.uidByte, mfrc522.uid.size);
      MFRC522::PICC_Type piccType = mfrc522.PICC_GetType(mfrc522.uid.sak);
      Serial.println(mfrc522.PICC_GetTypeName(piccType));

      MFRC522::StatusCode status;
      byte buffer[18];
      byte size = sizeof(buffer);
      
      status = (MFRC522::StatusCode) mfrc522.MIFARE_Read(RFID_BLOCK_ADDR, buffer, &size);
      if (status != MFRC522::STATUS_OK) {
        Serial.print("MIFARE_Read() failed: ");
        Serial.println(mfrc522.GetStatusCodeName(status));
      } else {
        Serial.println("Data in block : ");
        dump_byte_array(buffer, 16);
        if (memcmp(RFID_KEY, buffer, sizeof(RFID_KEY)) == 0) {
          unlockDoor("RFID");
        } else {
          Serial.println("Wrong Key");
        }
      }
    }
  }
  
  Blynk.run();
  timer.run();
} 
